#define MidiInChannel 10

#include "MidiInput.h"
#include "MidiDefs.h"
#include "system.h"
#include "TimerSynth.h"

// HS2 PLL x8

//Macros for Configuration Fuse Registers:
//Invoke macros to set up  device configuration fuse registers.
//The fuses will select the oscillator source, power-up timers, watch-dog
//timers, BOR characteristics etc. The macros are defined within the device
//header files. The configuration fuse registers reside in Flash memory.
//_FOSC(CSW_FSCM_OFF & FRC_PLL16);  //Run this project using an external crystal
                                //routed via the PLL in 16x multiplier mode
                                //For the 7.3728 MHz crystal we will derive a
                                //throughput of 7.3728e+6*16/4 = 29.4 MIPS(Fcy)
                                //,~33.9 nanoseconds instruction cycle time(Tcy).
_FWDT(WDT_OFF);                 //Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_OFF& BORV_27 & PWRT_64 & MCLR_EN & PWMxL_ACT_HI & PWMxH_ACT_LO);   //Enable MCLR reset pin and turn off the
                                //power-up timers.
_FGS(CODE_PROT_OFF);            //Disable Code Protection

volatile char lastNote;
volatile char CC69;
volatile char CC70;
volatile char CC20;
volatile char CC21;
volatile char CC64;
volatile char CC65;


// RB3 is Hammer
// RB2 is Damper

void initParams(void);

int main(void)
{
	CC69 = 0;
	CC70 = 0;
	CC20 = 33;
	CC21 = 50;
	CC64 = 0;
	CC65 = 126;
	NoteOffset1 = CC20-40;	
	Flageolet1 = 1;
	MISetupSerial();
	initParams();
	InitTimer23();
	DutyCycle1 = 64*128;
	DutyCycle2 = 64*128;

	TRISB = 0;
	//TRISBbits.TRISB2 = 0;
	//TRISBbits.TRISB3 = 0;

	while(1)
	{
		MIUartRXPollHandler();
//		U1TXREG = 0x56;
	}
}



void ScheduleHammer(unsigned char m) {
// timer 1
// max: 64 ms
// tick : 0.5 ms
	
	PORTBbits.RB3 = 1;

	T1CON = 0;// ; Stops the Timer1 and reset control reg, 
	T1CONbits.TCKPS = 3; //prescaler = 256, 8.5 us
	TMR1 = 0; ; //Clear contents of the timer register
	PR1 = (m + 1) * 58;
	IPC0bits.T1IP = 3; 
	IFS0bits.T1IF = 0;
	IEC0bits.T1IE = 1;
	T1CONbits.TON = 1;
}

void ScheduleDamper(unsigned char m) {

	PORTBbits.RB2 = 1;	
// 20 ms per unit
// 20ms .. 2.4 s range

// prescaler 1:1 means .033 us tick 
// Timer 4/5 32bit cascade
// MSWord is 2.16 ms/unit

	T4CON = 0;
	T5CON = 0;
	TMR5 = 0; 
	TMR4 = 0; 
	PR4 = 0;
	PR5 = (m + 1) * 10;

	IPC5bits.T5IP = 3; 
	IFS1bits.T5IF = 0;
	IEC1bits.T5IE = 1;
	T4CONbits.T32 = 1;
	T4CONbits.TON = 1;

	/*
CLR T2CON ; Stops any 16/32-bit Timer2 operation
CLR T3CON ; Stops any 16-bit Timer3 operation
CLR TMR3 ; Clear contents of the Timer3 timer register
CLR TMR2 ; Clear contents of the Timer2 timer register
MOV #0xFFFF, w0 ; Load the Period Register 3
MOV w0, PR3 ; with the value 0xFFFF
MOV w0, PR2 ; Load the Period Register2 with value 0xFFFF
BSET IPC1, #T3IP0 ; Setup Timer3 interrupt for
BCLR IPC1, #T3IP1 ; desired priority level
BCLR IPC1, #T3IP2 ; (this example assigns level 1 priority)
BCLR IFS0, #T3IF ; Clear the Timer3 interrupt status flag
BSET IEC0, #T3IE ; Enable Timer3 interrupts
BSET T2CON, #T32 ; Enable 32-bit Timer operation
BSET T2CON, #TON ; Start 32-bit timer with prescaler
; settings at 1:1 and clock source set to
; the internal instruction cycle
	*/
}

void MidiInNoteOn(void)
{
	if (!MidiByte2)
		MidiInNoteOff();
	else
	{	
		char ch;
		ch = MidiByte0 & 0x0F;
		if (ch == MidiInChannel)
		{
//			lastNote = MidiByte1;
			if (CC69)
			{
				if ((MidiByte1>=36) && (MidiByte1<=60)) // magnet excitation
				{
					PORTBbits.RB2 = 0; // damper off
					Osc1NoteOn(MidiByte1);
//					DutyCycle1 = (MidiByte2+1)*128;
					Osc1Calc();
				}
				if ((MidiByte1>=24) && (MidiByte1<=35)) // hammer excitation
				{
					ScheduleHammer(MidiByte2);
				}
			}
		}
	}
}

void MidiInNoteOff(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{		
		if ((MidiByte1>=36) && (MidiByte1<=60))
		{	
			Osc1NoteOff();
			if (CC64 == 0) {
				if (CC65 != 127) {
					unsigned char m;
					m = CC65>MidiByte2?CC65:MidiByte2;
					ScheduleDamper(m);
				} else {
					PORTBbits.RB2 = 1;
				}
			}
		}
	}
}

void initParams(void)
{
	MidiByte0 = 0;
	lastNote = 0;

//	MidiByte1 = MIDI_C_PORTA;
//	MidiByte2 = 0;
//	MidiInControlChange();

}

void MidiInControlChange(void)
{
	char ch = MidiByte0 & 0x0F;
	if (ch == MidiInChannel)
	{
//		int p;
		switch(MidiByte1)
		{
			case 3: // pwm
				DutyCycle1 = (MidiByte2+1)*256;
				Osc1Calc();
				break;
			case 4: // pwm
				DutyCycle2 = (MidiByte2+1)*256;
				Osc1Calc();
				break;
			case 20:
				NoteOffset1 = MidiByte2-40;
				break;
			case 22:
				if (MidiByte2 == 0)
					MidiByte2 = 1;
				if (MidiByte2 > 64)
					MidiByte2 = 64;
				Flageolet1 = MidiByte2;
				Osc1Calc();
				break;
			case 24:
				pitchBend1 = 2*(MidiByte2 - 64);
				Osc1Calc();
				break;
			case 64:
				CC64 = MidiByte2;
				break;
			case 65:
				CC65 = MidiByte2;
				break;
			case 69:
				CC69 = MidiByte2;
				break;
			case 70: 
				CC70 = MidiByte2;
				break;
			case MIDI_C_MODULATION:
				break;	
			case MIDI_C_CHORUS_DEPTH:
				break;
			case MIDI_C_EFFECT_DEPTH: 
				break;
			case MIDI_C_TREMELO_DEPTH:
				break;
			case MIDI_C_GENERAL_1: 
				break;
			case MIDI_C_GENERAL_2:
				break;
			case MIDI_C_GENERAL_3:
				break;
			case MIDI_C_GENERAL_4:
				break;
			case MIDI_C_GENERAL_5:
				break;
			case MIDI_C_ALL_NOTES_OFF:
				{
					Osc1NoteOff();
					pitchBend1 = 0;
				}
				break;
			default:
				break;
		}
	}
}

void MidiInPolyPressure(void)
{
}

void MidiInPitchBend(void)
{
}

void MidiInSysexStart(void)
{
}

void MidiInMTC(void)
{
}

void MidiInSongPosition(void)
{
}

void MidiInSongSelect(void)
{
}

void MidiInTuneRequest(void)
{
}

void MidiInSysexEnd(void)
{
}

void MidiInTimingClock(void)
{
}

void MidiInMeasureEnd(void)
{
}

void MidiInStart(void)
{
}

void MidiInContinue(void)
{
}

void MidiInStop(void)
{
}

void MidiInActiveSense(void)
{
}

void MidiInReset(void)
{
}

void MidiInProgramChange(void)
{
}

void MidiInChannelPressure(void)
{
}

void _ISR _T1Interrupt( void ) { // hammer
	PORTBbits.RB3 = 0;
	IFS0bits.T1IF = 0;
	IEC0bits.T1IE = 0;
}

void _ISR _T5Interrupt( void ) { // damper
	PORTBbits.RB2 = 0;	
	IFS1bits.T5IF = 0;
	IEC1bits.T5IE = 0;
}


extern void __attribute__((__interrupt__)) _PWMInterrupt(void);

void __attribute__((__interrupt__)) _T1Interrupt( void );
void __attribute__((__interrupt__)) _T5Interrupt( void );
